//
//  DrawController.swift
//  MassageMe
//
//  Created by Megan Leszczynski on 11/25/16.
//  Copyright © 2016 Harry+Margi. All rights reserved.
//

import UIKit
import ChameleonFramework
import CoreBluetooth

// protocol
let draw_msg: UInt8 = 3

let left_col = 80
let left_mid_col = 125
let right_col = 220
let right_mid_col = 175
let first_row_hi = 150
let first_row_lo = 170
let second_row = 210
let third_row = 300
let fourth_row = 340

let reset_msg = 5

class DrawController: UIViewController, BluetoothSerialDelegate {

    var imageView = UIImageView(image: UIImage(named: "vestoutline2.png")!)
    var message: [UInt8] = [draw_msg]
    var running: Bool = false

    var disk1:UIButton?
    var disk2:UIButton?
    var disk3:UIButton?
    var disk4:UIButton?
    var disk5:UIButton?
    var disk6:UIButton?
    var disk7:UIButton?
    var disk8:UIButton?
    var disk9:UIButton?
    var disk10:UIButton?
    var disk11:UIButton?
    var disk12:UIButton?
    var disk13:UIButton?
    var disk14:UIButton?
    var disk15:UIButton?
    var disk16:UIButton?
    
    let startButton = UIButton(frame: CGRect(x: 100, y: 475, width: 250, height: 50))

    func makeDisk(x_val: Int, y_val: Int, tag: Int)-> UIButton {
        let disk = UIButton(frame: CGRect(x: x_val, y: y_val, width: 20, height: 20))
        disk.layer.cornerRadius = disk.bounds.size.width / 2.0;
        disk.layer.borderColor = HexColor("8b008B").CGColor
        disk.layer.borderWidth = 1;
        disk.backgroundColor = .blackColor()
        disk.tag = tag
        disk.addTarget(self, action: #selector(diskPushed), forControlEvents: .TouchUpInside)
        return disk
    }
    
    override func viewDidLoad() {
        self.view.backgroundColor = UIColor.blackColor()

        imageView.frame = CGRectMake(0,90,280,350)
        imageView.center.x = self.view.center.x
        self.view.addSubview(imageView)
        
        disk1 = makeDisk(left_col, y_val: first_row_hi, tag: 0) // u
        disk2 = makeDisk(left_col, y_val: second_row, tag: 2)
        disk3 = makeDisk(left_col, y_val: third_row, tag: 8) // l
        disk4 = makeDisk(left_col, y_val: fourth_row, tag: 10)
        disk5 = makeDisk(left_mid_col, y_val: first_row_lo, tag: 1) // u
        disk6 = makeDisk(left_mid_col, y_val: second_row, tag: 3)
        disk7 = makeDisk(left_mid_col, y_val: third_row, tag: 9) // l
        disk8 = makeDisk(left_mid_col, y_val: fourth_row, tag: 11)
        disk9 = makeDisk(right_col, y_val: first_row_hi, tag: 5) // u
        disk10 = makeDisk(right_col, y_val: second_row, tag: 7)
        disk11 = makeDisk(right_col, y_val: third_row, tag: 13) // l
        disk12 = makeDisk(right_col, y_val: fourth_row, tag: 15)
        disk13 = makeDisk(right_mid_col, y_val: first_row_lo, tag: 4) // u
        disk14 = makeDisk(right_mid_col, y_val: second_row, tag: 6)
        disk15 = makeDisk(right_mid_col, y_val: third_row, tag: 12) // l
        disk16 = makeDisk(right_mid_col, y_val: fourth_row, tag: 14)
        
        self.view.addSubview(disk1!)
        self.view.addSubview(disk2!)
        self.view.addSubview(disk3!)
        self.view.addSubview(disk4!)
        self.view.addSubview(disk5!)
        self.view.addSubview(disk6!)
        self.view.addSubview(disk7!)
        self.view.addSubview(disk8!)
        self.view.addSubview(disk9!)
        self.view.addSubview(disk10!)
        self.view.addSubview(disk11!)
        self.view.addSubview(disk12!)
        self.view.addSubview(disk13!)
        self.view.addSubview(disk14!)
        self.view.addSubview(disk15!)
        self.view.addSubview(disk16!)
        
        startButton.backgroundColor = UIColor.blackColor()
        startButton.setTitleColor(HexColor("8b008B"), forState: UIControlState.Normal)
        startButton.layer.cornerRadius = 25
        startButton.layer.borderColor = HexColor("8b008B").CGColor
        startButton.layer.borderWidth = 1;
        startButton.clipsToBounds = true
        startButton.setTitle("START MASSAGE", forState: UIControlState.Normal)
        startButton.addTarget(self, action: #selector(startPushed), forControlEvents: .TouchUpInside)
        startButton.center.x = self.view.center.x
        self.view.addSubview(startButton)
        
        self.becomeFirstResponder()
        
        serial.delegate = self
    }
    
    func reset(){
        message = [draw_msg] // reset msg
        disk1!.backgroundColor = .blackColor()
        disk2!.backgroundColor = .blackColor()
        disk3!.backgroundColor = .blackColor()
        disk4!.backgroundColor = .blackColor()
        disk5!.backgroundColor = .blackColor()
        disk6!.backgroundColor = .blackColor()
        disk7!.backgroundColor = .blackColor()
        disk8!.backgroundColor = .blackColor()
        disk9!.backgroundColor = .blackColor()
        disk10!.backgroundColor = .blackColor()
        disk11!.backgroundColor = .blackColor()
        disk12!.backgroundColor = .blackColor()
        disk13!.backgroundColor = .blackColor()
        disk14!.backgroundColor = .blackColor()
        disk15!.backgroundColor = .blackColor()
        disk16!.backgroundColor = .blackColor()
    }

    override func motionEnded(motion: UIEventSubtype, withEvent event: UIEvent?) {
        if(event!.subtype == UIEventSubtype.MotionShake) {
            reset()
        }
    }
    
    func diskPushed(sender: UIButton){
        if (sender.backgroundColor == .blackColor()){
            message.append(UInt8(sender.tag)) // add button to list of pressed buttons
            sender.backgroundColor = HexColor("8b008b")
        }
        else{
           let index = message[1...message.endIndex-1].indexOf(UInt8(sender.tag))
            if index != nil {
                message.removeAtIndex(index!)
            }
            sender.backgroundColor = .blackColor()
        }
    }
    
    func setDisks(value: Bool){
        disk1!.enabled = value
        disk2!.enabled = value
        disk3!.enabled = value
        disk4!.enabled = value
        disk5!.enabled = value
        disk6!.enabled = value
        disk7!.enabled = value
        disk8!.enabled = value
        disk9!.enabled = value
        disk10!.enabled = value
        disk11!.enabled = value
        disk12!.enabled = value
        disk13!.enabled = value
        disk14!.enabled = value
        disk15!.enabled = value
        disk16!.enabled = value
    }
    
    func resetStart(){
        running = false
        startButton.setTitle("START MASSAGE", forState: .Normal)
        startButton.backgroundColor = UIColor.blackColor()
        startButton.setTitleColor(HexColor("8b008B"), forState: UIControlState.Normal)
        reset()
        setDisks(true)
    }
    
    func startPushed(sender: UIButton) {
        let length = message.count
        if length == 1{
            // haven't entered pattern yet
            let alertController = UIAlertController(title: nil, message:
                "Draw a pattern!", preferredStyle: UIAlertControllerStyle.Alert)
            alertController.addAction(UIAlertAction(title: "Dismiss", style: UIAlertActionStyle.Default,handler: nil))
            
            self.presentViewController(alertController, animated: true, completion: nil)
            return
        }
        if serial.isReady && !running{
            message.append(eot) // end of transmission
            print(message)
            serial.sendBytesToDevice(message)
            running = true
            // change to stop massage option
            sender.backgroundColor = HexColor("8b008B")
            sender.setTitleColor(.whiteColor(), forState: .Normal)
            sender.setTitle("STOP MASSAGE", forState: .Normal)
            setDisks(false)
        }
        else if (running){
            resetStart()
            serial.sendBytesToDevice([stop_msg, eot])
            print([stop_msg, eot])
        }
    }
    
    /// Called when a message is received
    func serialDidReceiveBytes(bytes: [UInt8]){
        if bytes == [UInt8(reset_msg)] && running {
            resetStart()
        }
//        print(bytes)
    }
    
    func serialDidChangeState(newState: CBCentralManagerState){
    }
    
    /// Called when a peripheral disconnected
    func serialDidDisconnect(peripheral: CBPeripheral, error: NSError?){
    }
}
